Index: channels/chan_sip.c
===================================================================
--- channels/chan_sip.c	(revision 41573)
+++ channels/chan_sip.c	(working copy)
@@ -882,6 +882,7 @@
 		AST_STRING_FIELD(peername);	/*!< [peer] name, not set if [user] */
 		AST_STRING_FIELD(authname);	/*!< Who we use for authentication */
 		AST_STRING_FIELD(uri);		/*!< Original requested URI */
+		AST_STRING_FIELD(ruri);         /*!< SIP Request-URI */
 		AST_STRING_FIELD(okcontacturi);	/*!< URI from the 200 OK on INVITE */
 		AST_STRING_FIELD(peersecret);	/*!< Password */
 		AST_STRING_FIELD(peermd5secret);
@@ -11064,6 +11065,10 @@
 static int function_sipchaninfo_read(struct ast_channel *chan, char *cmd, char *data, char *buf, size_t len)
 {
 	struct sip_pvt *p;
+	AST_DECLARE_APP_ARGS(args,
+                AST_APP_ARG(item);
+                AST_APP_ARG(param);
+        );
 
 	*buf = 0;
 	
@@ -11087,6 +11092,9 @@
 		return -1;
 	}
 
+	/* parse data into args --> args.item, args.param */
+	AST_STANDARD_APP_ARGS(args, data);
+
 	if (!strcasecmp(data, "peerip")) {
 		ast_copy_string(buf, p->sa.sin_addr.s_addr ? ast_inet_ntoa(p->sa.sin_addr) : "", len);
 	} else  if (!strcasecmp(data, "recvip")) {
@@ -11104,6 +11112,50 @@
 			ast_copy_string(buf, "0", sizeof("0"));
 		else    /* T38 is offered or enabled in this call */
 			ast_copy_string(buf, "1", sizeof("1"));
+	} else if (!strcasecmp(args.item, "ruri")) {
+		char *pat_start = ";";
+        	char *pat_end = "=([^;]+)";
+        	char *pattern = NULL;
+        	regmatch_t pmatch[2];
+        	regex_t re;		
+
+		if (!args.param) {
+			/* no data means we return full R-URI */
+			ast_copy_string(buf, p->ruri, len);
+        	} else {
+			/* compile regex pattern: ;<data>=([^;]+) */
+        		if (!(pattern = (char *)ast_calloc(strlen(pat_start) + strlen(pat_end) + strlen(args.param) + 1,
+                		                           sizeof(char)))) {
+                		ast_channel_unlock(chan);
+                		return -1;
+        		}
+        		strcpy(pattern, pat_start);
+        		strncat(pattern, args.param, strlen(args.param));
+        		strcat(pattern, pat_end);
+        		if (regcomp(&re, pattern, REG_EXTENDED) != 0) {
+                		free(pattern);
+                		regfree(&re);
+                		ast_channel_unlock(chan);
+                		return -1;
+			}
+
+			/* apply pattern to p->ruri */
+        		if (regexec(&re, p->ruri, (size_t) 2, pmatch, 0) == 0) {
+                		/* pattern found, copy substring 1 into buf */
+				ast_copy_string(buf, 
+						p->ruri + pmatch[1].rm_so, 
+						pmatch[1].rm_eo - pmatch[1].rm_so + 1);
+				regfree(&re);
+        			free(pattern);
+        		} else {
+                		/* pattern not found */
+                		regfree(&re);
+                		free(pattern);
+                		ast_channel_unlock(chan);
+                		return -1;
+        		}
+        	}
+
 	} else {
 		ast_channel_unlock(chan);
 		return -1;
@@ -11117,13 +11169,14 @@
 static struct ast_custom_function sipchaninfo_function = {
 	.name = "SIPCHANINFO",
 	.synopsis = "Gets the specified SIP parameter from the current channel",
-	.syntax = "SIPCHANINFO(item)",
+	.syntax = "SIPCHANINFO(<item>[,<param>])",
 	.read = function_sipchaninfo_read,
 	.desc = "Valid items are:\n"
 	"- peerip                The IP address of the peer.\n"
 	"- recvip                The source IP address of the peer.\n"
 	"- from                  The URI from the From: header.\n"
 	"- uri                   The URI from the Contact: header.\n"
+	"- ruri[,param]          The Request-URI from the SIP request, <param> specifies SIP URI parameter value.\n"
 	"- useragent             The useragent.\n"
 	"- peername              The name of the peer.\n"
 	"- t38passthrough        1 if T38 is offered or enabled in this channel, otherwise 0\n"
@@ -12643,6 +12696,9 @@
 		}
 	}
 
+        /* save Request-URI in p->ruri */
+        ast_string_field_set(p, ruri, e);
+
 	/* Check if this is a loop */
 	if (ast_test_flag(&p->flags[0], SIP_OUTGOING) && p->owner && (p->owner->_state != AST_STATE_UP)) {
 		/* This is a call to ourself.  Send ourselves an error code and stop
